<?php

class InventoryLib {

/**
 * Check the given model has enough stock to handle the requirement.
 *
 * @param array $data The model data - if it actsAs Inventories then Inventory element should be available.
 * @param int   $quantity The quantity we want.
 * @return bool.
 */
	public static function hasEnoughStock($data, $quantity) {
		if (isset($data['Inventory']['stock']) && $data['Inventory']['stock'] >= $quantity) {
			return true;
		}

		// so not enough stock, check to see if we have actually disallowed selling
		if (! is_array($data['Inventory']['oos_action'])) {
			$data['Inventory']['oos_action'] = array();
		}

		if (
			! in_array('stop', $data['Inventory']['oos_action']) ||
			in_array('delayed', $data['Inventory']['oos_action'])
		) {
			return true;
		}

		return false;
	}

/**
 * Synchronise the stock of multiple items. Inventories are found that have the same data as the
 * inventory to synchronise as defined in the config.
 *
 * @param Model $Model   The model that is being updated.
 * @param int   $modelId The id of the model that is being updated.
 * @return bool True if the stock was synchronised successfully or no changes have occurred, false otherwise.
 */
	public function syncStock($Model, $modelId) {
		$syncOptions = Configure::read('EvInventory.synchronise');

		if (empty($syncOptions)) {
			//There are no synchronisation options so return without changes.
			return true;
		}

		$Inventory = EvClassRegistry::init('EvInventory.Inventory');

		//Get the inventory that we are attempting to synchronise.
		$modelName = EvClassRegistry::getNameFromModel($Model);
		$syncInventory = $Inventory->getInventory($modelName, $modelId);

		if (empty($syncOptions[$Model->alias])) {
			return true;
		}

		//Extract the values from the inventory we are syncing so that they can be used to find
		//other inventories. If some of the data is missing then stop syncing as the correct
		//inventories won't be able to be found.
		$canSync = true;
		$syncConditions = [];
		foreach ($syncOptions[$Model->alias] as $syncField) {
			if (empty($syncInventory[$Model->alias][$syncField])) {
				$canSync = false;
				break;
			}

			$syncConditions[$Model->alias . '.' . $syncField] = $syncInventory[$Model->alias][$syncField];
		}

		if (!$canSync) {
			return true;
		}

		//Add the join condition.
		$syncConditions[] = $Model->alias . '.' . $Model->primaryKey . ' = ' . $Inventory->alias . '.model_id';

		$syncInventoriesIds = $Inventory->find(
			'list',
			[
				'fields' => [
					'id',
					'id',
				],
				'joins' => [
					[
						'table' => $Model->tablePrefix . $Model->table,
						'alias' => $Model->alias,
						'conditions' => $syncConditions,
					],
				],
				'conditions' => [
					$Inventory->alias . '.id !=' => $syncInventory[$Inventory->alias]['id'],
				]
			]
		);

		if (empty($syncInventoriesIds)) {
			return true;
		}

		return $Inventory->updateAll(
			[
				$Inventory->alias . '.stock' => $syncInventory[$Inventory->alias]['stock'],
			],
			[
				$Inventory->alias . '.id' => $syncInventoriesIds,
			]
		);
	}
}
